@@ -104,8 +104,9 @@ typedef struct cm_buf {
 #define CM_BUF_INLRU	0x10	/* in lru queue */
 #define CM_BUF_ERROR	0x20	/* something went wrong on delayed write */
 #define CM_BUF_WAITING	0x40	/* someone's waiting for a flag to change */
-
+#define CM_BUF_INDL     0x80    /* in the dirty list */
 #define CM_BUF_EOF	0x100	/* read 0 bytes; used for detecting EOF */
+#define CM_BUF_REDIR    0x200   /* buffer held by the redirector */
 
 typedef struct cm_buf_ops {
     long (*Writep)(void *, osi_hyper_t *, long, long, struct cm_user *,
@@ -124,15 +125,30 @@ extern void buf_Shutdown(void);
 
 extern long buf_CountFreeList(void);
 
+#ifdef DEBUG_REFCOUNT
+extern void buf_ReleaseDbg(cm_buf_t *, char *, long);
+
+extern void buf_HoldDbg(cm_buf_t *, char *, long);
+
+extern void buf_ReleaseLockedDbg(cm_buf_t *, afs_uint32, char *, long);
+
+extern void buf_HoldLockedDbg(cm_buf_t *, char *, long);
+
+#define buf_Release(bufp) buf_ReleaseDbg(bufp, __FILE__, __LINE__)
+#define buf_Hold(bufp)    buf_HoldDbg(bufp, __FILE__, __LINE__)
+#define buf_ReleaseLocked(bufp, lock) buf_ReleaseLockedDbg(bufp, lock, __FILE__, __LINE__)
+#define buf_HoldLocked(bufp) buf_HoldLockedDbg(bufp, __FILE__, __LINE__)
+#else
 extern void buf_Release(cm_buf_t *);
 
 extern void buf_Hold(cm_buf_t *);
 
-extern void buf_WaitIO(cm_scache_t *, cm_buf_t *);
-
 extern void buf_ReleaseLocked(cm_buf_t *, afs_uint32);
 
 extern void buf_HoldLocked(cm_buf_t *);
+#endif
+
+extern void buf_WaitIO(cm_scache_t *, cm_buf_t *);
 
 extern cm_buf_t *buf_FindLocked(struct cm_scache *, osi_hyper_t *);
 
@@ -150,7 +166,7 @@ extern long buf_CleanAsync(cm_buf_t *, cm_req_t *);
 
 extern void buf_CleanWait(cm_scache_t *, cm_buf_t *, afs_uint32 locked);
 
-extern void buf_SetDirty(cm_buf_t *, afs_uint32 offset, afs_uint32 length);
+extern void buf_SetDirty(cm_buf_t *, afs_uint32 offset, afs_uint32 length, cm_user_t *);
 
 extern long buf_CleanAndReset(void);
 
@@ -187,6 +203,8 @@ extern long buf_CleanDirtyBuffers(cm_scache_t *scp);
 
 extern long buf_ForceDataVersion(cm_scache_t * scp, afs_uint64 fromVersion, afs_uint64 toVersion);
 
+extern int cm_DumpBufHashTable(FILE *outputFile, char *cookie, int lock);
+
 /* error codes */
 #define CM_BUF_EXISTS	1	/* buffer exists, and shouldn't */
 #endif /*  _BUF_H__ENV_ */
