@@ -111,18 +111,20 @@ IsWindowsModule(const char * name)
  * If the caller wants the "real" cell name, it puts a non-null pointer in
  * newCellNamep.  Anomaly:  if cellNamep is ambiguous, we may modify
  * newCellNamep but return an error code.
+ *
+ * newCellNamep is required to be CELL_MAXNAMELEN in size.
  */
 long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
                        cm_configProc_t *procp, void *rockp)
 {
-    char wdir[257];
+    char wdir[MAX_PATH]="";
     FILE *tfilep = NULL, *bestp, *tempp;
     char *tp;
     char lineBuffer[257];
     struct hostent *thp;
     char *valuep;
     struct sockaddr_in vlSockAddr;
-    int inRightCell;
+    int inRightCell = 0;
     int foundCell = 0;
     long code;
     int tracking = 1, partial = 0;
@@ -131,7 +133,8 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
 	return -3;
 
     cm_GetCellServDB(wdir, sizeof(wdir));
-    tfilep = fopen(wdir, "r");
+    if (*wdir)
+        tfilep = fopen(wdir, "r");
 
     if (!tfilep) 
         return -2;
@@ -143,12 +146,11 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
 	     osi_LogSaveString(afsd_logp,wdir));
 #endif
     /* have we seen the cell line for the guy we're looking for? */
-    inRightCell = 0;
     while (1) {
         tp = fgets(lineBuffer, sizeof(lineBuffer), tfilep);
         if (tracking)
 	    (void) fgets(lineBuffer, sizeof(lineBuffer), bestp);
-        if (	tp == NULL) {
+        if (tp == NULL) {
 	    if (feof(tfilep)) {
 		/* hit EOF */
 		if (partial) {
@@ -172,15 +174,21 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
         }	
 
         /* turn trailing cr or lf into null */
-        tp = strchr(lineBuffer, '\r');
+        tp = strrchr(lineBuffer, '\r');
         if (tp) *tp = 0;
-        tp = strchr(lineBuffer, '\n');
+        tp = strrchr(lineBuffer, '\n');
         if (tp) *tp = 0;
 
 	/* skip blank lines */
         if (lineBuffer[0] == 0) continue;
 
         if (lineBuffer[0] == '>') {
+            if (inRightCell) {
+                fclose(tfilep);
+                fclose(bestp);
+                return(foundCell ? 0 : -6);
+            }
+
 	    /* trim off at white space or '#' chars */
             tp = strchr(lineBuffer, ' ');
             if (tp) *tp = 0;
@@ -193,7 +201,8 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
             if (stricmp(lineBuffer+1, cellNamep) == 0) {
 		/* found the cell we're looking for */
 		if (newCellNamep) {
-		    strcpy(newCellNamep, lineBuffer+1);
+		    strncpy(newCellNamep, lineBuffer+1,CELL_MAXNAMELEN);
+                    newCellNamep[CELL_MAXNAMELEN-1] = '\0';
                     strlwr(newCellNamep);
                 }
                 inRightCell = 1;
@@ -211,7 +220,8 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
 		    return -5;
 		}
 		if (newCellNamep) {
-		    strcpy(newCellNamep, lineBuffer+1);
+		    strncpy(newCellNamep, lineBuffer+1,CELL_MAXNAMELEN);
+                    newCellNamep[CELL_MAXNAMELEN-1] = '\0';
                     strlwr(newCellNamep);
                 }
 		inRightCell = 0;
@@ -254,28 +264,29 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
                     foundCell = 1;
 		}
                 if (!thp) {
-                    long ip_addr;
+                    afs_uint32 ip_addr;
 		    int c1, c2, c3, c4;
-		    char aname[241] = "";                    
                     
                     /* Since there is no gethostbyname() data 
 		     * available we will read the IP address
 		     * stored in the CellServDB file
                      */
-                    code = sscanf(lineBuffer, "%d.%d.%d.%d #%s",
-                                   &c1, &c2, &c3, &c4, aname);
-                    tp = (char *) &ip_addr;
-                    *tp++ = c1;
-                    *tp++ = c2;
-                    *tp++ = c3;
-                    *tp++ = c4;
-                    memcpy(&vlSockAddr.sin_addr.s_addr, &ip_addr,
-                            sizeof(long));
-                    vlSockAddr.sin_family = AF_INET;
-                    /* sin_port supplied by connection code */
-                    if (procp)
-                        (*procp)(rockp, &vlSockAddr, valuep);
-                    foundCell = 1;
+                    code = sscanf(lineBuffer, " %d.%d.%d.%d",
+                                   &c1, &c2, &c3, &c4);
+                    if (code == 4) {
+                        tp = (char *) &ip_addr;
+                        *tp++ = c1;
+                        *tp++ = c2;
+                        *tp++ = c3;
+                        *tp++ = c4;
+                        memcpy(&vlSockAddr.sin_addr.s_addr, &ip_addr,
+                                sizeof(long));
+                        vlSockAddr.sin_family = AF_INET;
+                        /* sin_port supplied by connection code */
+                        if (procp)
+                            (*procp)(rockp, &vlSockAddr, valuep);
+                        foundCell = 1;
+                    }
                 }
             }
         }	/* a vldb line */
@@ -285,6 +296,7 @@ long cm_SearchCellFile(char *cellNamep, char *newCellNamep,
     return (foundCell) ? 0 : -11;
 }
 
+/* newCellNamep is required to be CELL_MAXNAMELEN in size */
 long cm_SearchCellByDNS(char *cellNamep, char *newCellNamep, int *ttl,
                cm_configProc_t *procp, void *rockp)
 {
@@ -310,7 +322,8 @@ long cm_SearchCellByDNS(char *cellNamep, char *newCellNamep, int *ttl,
             if (procp)
                 (*procp)(rockp, &vlSockAddr, cellHostNames[i]);
             if (newCellNamep) {
-                strcpy(newCellNamep,cellNamep);
+                strncpy(newCellNamep,cellNamep,CELL_MAXNAMELEN);
+                newCellNamep[CELL_MAXNAMELEN-1] = '\0';
                 strlwr(newCellNamep);
             }
         }
@@ -329,19 +342,21 @@ long cm_SearchCellByDNS(char *cellNamep, char *newCellNamep, int *ttl,
  */
 long cm_GetCellServDB(char *cellNamep, afs_uint32 len)
 {
-    int tlen;
+    size_t tlen;
     
     cm_GetConfigDir(cellNamep, len);
 
     /* add trailing backslash, if required */
     tlen = (int)strlen(cellNamep);
-    if (cellNamep[tlen-1] != '\\') {
-        strncat(cellNamep, "\\", len);
+    if (tlen) {
+        if (cellNamep[tlen-1] != '\\') {
+            strncat(cellNamep, "\\", len);
+            cellNamep[len-1] = '\0';
+        }
+        
+        strncat(cellNamep, AFS_CELLSERVDB, len);
         cellNamep[len-1] = '\0';
     }
-        
-    strncat(cellNamep, AFS_CELLSERVDB, len);
-    cellNamep[len-1] = '\0';
     return 0;
 }
 
@@ -368,16 +383,16 @@ long cm_GetRootCellName(char *cellNamep)
 
 cm_configFile_t *cm_CommonOpen(char *namep, char *rwp)
 {
-    char wdir[256];
-    FILE *tfilep;
+    char wdir[MAX_PATH]="";
+    FILE *tfilep = NULL;
 
     cm_GetConfigDir(wdir, sizeof(wdir));
-
-    strncat(wdir, namep, sizeof(wdir));
-    wdir[sizeof(wdir)-1] = '\0';
+    if (*wdir) {
+        strncat(wdir, namep, sizeof(wdir));
+        wdir[sizeof(wdir)-1] = '\0';
         
-    tfilep = fopen(wdir, rwp);
-
+        tfilep = fopen(wdir, rwp);
+    }
     return ((cm_configFile_t *) tfilep);        
 }	
 
@@ -512,8 +527,8 @@ long cm_AppendNewCellLine(cm_configFile_t *filep, char *linep)
 
 long cm_CloseCellFile(cm_configFile_t *filep)
 {
-    char wdir[260];
-    char sdir[260];
+    char wdir[MAX_PATH];
+    char sdir[MAX_PATH];
     long code;
     long closeCode;
     closeCode = fclose((FILE *)filep);
