@@ -10,11 +10,9 @@
 #include <afs/param.h>
 #include <afs/stds.h>
 
-#ifndef DJGPP
 #include <windows.h>
 #include <winsock2.h>
 #include <nb30.h>
-#endif /* !DJGPP */
 #ifdef COMMENT
 #include <malloc.h>
 #endif
@@ -28,7 +26,6 @@
 extern void afsi_log(char *pattern, ...);
 #endif
 
-osi_mutex_t cm_bufGetMutex;
 #ifdef AFS_FREELANCE_CLIENT
 extern osi_mutex_t cm_Freelance_Lock;
 #endif
@@ -64,7 +61,7 @@ long cm_BufWrite(void *vscp, osi_hyper_t *offsetp, long length, long flags,
     osi_hyper_t thyper;
     AFSVolSync volSync;
     AFSFid tfid;
-    struct rx_call *callp;
+    struct rx_call *rxcallp;
     struct rx_connection *rxconnp;
     osi_queueData_t *qdp;
     cm_buf_t *bufp;
@@ -155,7 +152,7 @@ long cm_BufWrite(void *vscp, osi_hyper_t *offsetp, long length, long flags,
 
     retry:
         rxconnp = cm_GetRxConn(connp);
-        callp = rx_NewCall(rxconnp);
+        rxcallp = rx_NewCall(rxconnp);
         rx_PutConnection(rxconnp);
 
 #ifdef AFS_LARGEFILES
@@ -163,7 +160,7 @@ long cm_BufWrite(void *vscp, osi_hyper_t *offsetp, long length, long flags,
             osi_Log4(afsd_logp, "CALL StartRXAFS_StoreData64 scp 0x%p, offset 0x%x:%08x, length 0x%x",
                      scp, biod.offset.HighPart, biod.offset.LowPart, nbytes);
 
-            code = StartRXAFS_StoreData64(callp, &tfid, &inStatus,
+            code = StartRXAFS_StoreData64(rxcallp, &tfid, &inStatus,
                                           biod.offset.QuadPart,
                                           nbytes,
                                           truncPos.QuadPart);
@@ -179,7 +176,7 @@ long cm_BufWrite(void *vscp, osi_hyper_t *offsetp, long length, long flags,
                 osi_Log4(afsd_logp, "CALL StartRXAFS_StoreData scp 0x%p, offset 0x%x:%08x, length 0x%x",
                          scp, biod.offset.HighPart, biod.offset.LowPart, nbytes);
 
-                code = StartRXAFS_StoreData(callp, &tfid, &inStatus,
+                code = StartRXAFS_StoreData(rxcallp, &tfid, &inStatus,
                                             biod.offset.LowPart, nbytes, truncPos.LowPart);
 		if (code)
 		    osi_Log1(afsd_logp, "CALL StartRXAFS_StoreData FAILURE, code 0x%x", code);
@@ -191,7 +188,7 @@ long cm_BufWrite(void *vscp, osi_hyper_t *offsetp, long length, long flags,
         osi_Log4(afsd_logp, "CALL StartRXAFS_StoreData scp 0x%p, offset 0x%x:%08x, length 0x%x",
                  scp, biod.offset.HighPart, biod.offset.LowPart, nbytes);
 
-        code = StartRXAFS_StoreData(callp, &tfid, &inStatus,
+        code = StartRXAFS_StoreData(rxcallp, &tfid, &inStatus,
                                     biod.offset.LowPart, nbytes, truncPos.LowPart);
 	if (code)
 	    osi_Log1(afsd_logp, "CALL StartRXAFS_StoreData FAILURE, code 0x%x", code);
@@ -215,7 +212,7 @@ long cm_BufWrite(void *vscp, osi_hyper_t *offsetp, long length, long flags,
                     wbytes = cm_data.buf_blockSize;
 
                 /* write out wbytes of data from bufferp */
-                temp = rx_Write(callp, bufferp, wbytes);
+                temp = rx_Write(rxcallp, bufferp, wbytes);
                 if (temp != wbytes) {
                     osi_Log3(afsd_logp, "rx_Write failed bp 0x%p, %d != %d",bufp,temp,wbytes);
                     code = -1;
@@ -229,13 +226,13 @@ long cm_BufWrite(void *vscp, osi_hyper_t *offsetp, long length, long flags,
 
         if (code == 0) {
             if (SERVERHAS64BIT(connp)) {
-                code = EndRXAFS_StoreData64(callp, &outStatus, &volSync);
+                code = EndRXAFS_StoreData64(rxcallp, &outStatus, &volSync);
                 if (code)
                     osi_Log2(afsd_logp, "EndRXAFS_StoreData64 FAILURE scp 0x%p code %lX", scp, code);
 		else
 		    osi_Log0(afsd_logp, "EndRXAFS_StoreData64 SUCCESS");
             } else {
-                code = EndRXAFS_StoreData(callp, &outStatus, &volSync);
+                code = EndRXAFS_StoreData(rxcallp, &outStatus, &volSync);
                 if (code)
                     osi_Log2(afsd_logp, "EndRXAFS_StoreData FAILURE scp 0x%p code %lX",scp,code);
 		else
@@ -243,7 +240,7 @@ long cm_BufWrite(void *vscp, osi_hyper_t *offsetp, long length, long flags,
             }
         }
 
-        code = rx_EndCall(callp, code);
+        code = rx_EndCall(rxcallp, code);
 
 #ifdef AFS_LARGEFILES
         if (code == RXGEN_OPCODE && SERVERHAS64BIT(connp)) {
@@ -324,7 +321,7 @@ long cm_StoreMini(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
     long code;
     osi_hyper_t truncPos;
     cm_conn_t *connp;
-    struct rx_call *callp;
+    struct rx_call *rxcallp;
     struct rx_connection *rxconnp;
     int require_64bit_ops = 0;
 
@@ -362,33 +359,33 @@ long cm_StoreMini(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
 
     retry:	
         rxconnp = cm_GetRxConn(connp);
-        callp = rx_NewCall(rxconnp);
+        rxcallp = rx_NewCall(rxconnp);
         rx_PutConnection(rxconnp);
 
 #ifdef AFS_LARGEFILES
         if (SERVERHAS64BIT(connp)) {
-            code = StartRXAFS_StoreData64(callp, &tfid, &inStatus,
+            code = StartRXAFS_StoreData64(rxcallp, &tfid, &inStatus,
                                           0, 0, truncPos.QuadPart);
         } else {
             if (require_64bit_ops) {
                 code = CM_ERROR_TOOBIG;
             } else {
-                code = StartRXAFS_StoreData(callp, &tfid, &inStatus,
+                code = StartRXAFS_StoreData(rxcallp, &tfid, &inStatus,
                                             0, 0, truncPos.LowPart);
             }
         }
 #else
-        code = StartRXAFS_StoreData(callp, &tfid, &inStatus,
+        code = StartRXAFS_StoreData(rxcallp, &tfid, &inStatus,
                                     0, 0, truncPos.LowPart);
 #endif
 
         if (code == 0) {
             if (SERVERHAS64BIT(connp))
-                code = EndRXAFS_StoreData64(callp, &outStatus, &volSync);
+                code = EndRXAFS_StoreData64(rxcallp, &outStatus, &volSync);
             else
-                code = EndRXAFS_StoreData(callp, &outStatus, &volSync);
+                code = EndRXAFS_StoreData(rxcallp, &outStatus, &volSync);
         }
-        code = rx_EndCall(callp, code);
+        code = rx_EndCall(rxcallp, code);
 
 #ifdef AFS_LARGEFILES
         if (code == RXGEN_OPCODE && SERVERHAS64BIT(connp)) {
@@ -485,7 +482,6 @@ long cm_ShutdownDCache(void)
 
 int cm_InitDCache(int newFile, long chunkSize, afs_uint64 nbuffers)
 {
-    lock_InitializeMutex(&cm_bufGetMutex, "buf_Get mutex");
     return buf_Init(newFile, &cm_bufOps, nbuffers);
 }
 
@@ -1094,7 +1090,6 @@ long cm_SetupFetchBIOD(cm_scache_t *scp, osi_hyper_t *offsetp,
      * sequence at a time.
      */
 
-    // lock_ObtainMutex(&cm_bufGetMutex);
     /* first hold all buffers, since we can't hold any locks in buf_Get */
     while (1) {
         /* stop at chunk boundary */
@@ -1107,7 +1102,6 @@ long cm_SetupFetchBIOD(cm_scache_t *scp, osi_hyper_t *offsetp,
 
         code = buf_Get(scp, &pageBase, &tbp);
         if (code) {
-            //lock_ReleaseMutex(&cm_bufGetMutex);
             lock_ObtainWrite(&scp->rw);
             cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
             return code;
@@ -1123,8 +1117,6 @@ long cm_SetupFetchBIOD(cm_scache_t *scp, osi_hyper_t *offsetp,
     /* reserve a chunk's worth of buffers if possible */
     reserving = buf_TryReserveBuffers(cm_chunkSize / cm_data.buf_blockSize);
 
-    // lock_ReleaseMutex(&cm_bufGetMutex);
-
     pageBase = *offsetp;
     collected = pageBase.LowPart & (cm_chunkSize - 1);
 
@@ -1346,7 +1338,7 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *userp
     cm_buf_t *tbufp;		        /* buf we're filling */
     osi_queueData_t *qdp;		/* q element we're scanning */
     AFSFid tfid;
-    struct rx_call *callp;
+    struct rx_call *rxcallp;
     struct rx_connection *rxconnp;
     cm_bulkIO_t biod;		/* bulk IO descriptor */
     cm_conn_t *connp;
@@ -1501,7 +1493,7 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *userp
             continue;
 
         rxconnp = cm_GetRxConn(connp);
-        callp = rx_NewCall(rxconnp);
+        rxcallp = rx_NewCall(rxconnp);
         rx_PutConnection(rxconnp);
 
 #ifdef AFS_LARGEFILES
@@ -1511,17 +1503,17 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *userp
             osi_Log4(afsd_logp, "CALL FetchData64 scp 0x%p, off 0x%x:%08x, size 0x%x",
                      scp, biod.offset.HighPart, biod.offset.LowPart, biod.length);
 
-            code = StartRXAFS_FetchData64(callp, &tfid, biod.offset.QuadPart, biod.length);
+            code = StartRXAFS_FetchData64(rxcallp, &tfid, biod.offset.QuadPart, biod.length);
 
             if (code == 0) {
-                temp = rx_Read(callp, (char *) &nbytes_hi, sizeof(afs_int32));
+                temp = rx_Read(rxcallp, (char *) &nbytes_hi, sizeof(afs_int32));
                 if (temp == sizeof(afs_int32)) {
                     nbytes_hi = ntohl(nbytes_hi);
                 } else {
                     nbytes_hi = 0;
-		    code = callp->error;
-                    rx_EndCall(callp, code);
-                    callp = NULL;
+		    code = rxcallp->error;
+                    rx_EndCall(rxcallp, code);
+                    rxcallp = NULL;
                 }
             }
         }
@@ -1531,16 +1523,16 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *userp
                 osi_Log0(afsd_logp, "Skipping FetchData.  Operation requires FetchData64");
                 code = CM_ERROR_TOOBIG;
             } else {
-                if (!callp) {
+                if (!rxcallp) {
                     rxconnp = cm_GetRxConn(connp);
-                    callp = rx_NewCall(rxconnp);
+                    rxcallp = rx_NewCall(rxconnp);
                     rx_PutConnection(rxconnp);
                 }
 
                 osi_Log3(afsd_logp, "CALL FetchData scp 0x%p, off 0x%x, size 0x%x",
                          scp, biod.offset.LowPart, biod.length);
 
-                code = StartRXAFS_FetchData(callp, &tfid, biod.offset.LowPart,
+                code = StartRXAFS_FetchData(rxcallp, &tfid, biod.offset.LowPart,
                                             biod.length);
 
                 SET_SERVERHASNO64BIT(connp);
@@ -1548,14 +1540,14 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *userp
         }
 
         if (code == 0) {
-            temp  = rx_Read(callp, (char *)&nbytes, sizeof(afs_int32));
+            temp  = rx_Read(rxcallp, (char *)&nbytes, sizeof(afs_int32));
             if (temp == sizeof(afs_int32)) {
                 nbytes = ntohl(nbytes);
                 FillInt64(length_found, nbytes_hi, nbytes);
                 if (length_found > biod.length) 
-                    code = (callp->error < 0) ? callp->error : -1;
+                    code = (rxcallp->error < 0) ? rxcallp->error : -1;
             } else {
-                code = (callp->error < 0) ? callp->error : -1;
+                code = (rxcallp->error < 0) ? rxcallp->error : -1;
             }
         }
         /* for the moment, nbytes_hi will always be 0 if code == 0
@@ -1564,19 +1556,19 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *userp
         osi_Log3(afsd_logp, "CALL FetchData scp 0x%p, off 0x%x, size 0x%x",
                  scp, biod.offset.LowPart, biod.length);
 
-        code = StartRXAFS_FetchData(callp, &tfid, biod.offset.LowPart,
+        code = StartRXAFS_FetchData(rxcallp, &tfid, biod.offset.LowPart,
                                     biod.length);
 
         /* now copy the data out of the pipe and put it in the buffer */
         if (code == 0) {
-            temp  = rx_Read(callp, (char *)&nbytes, sizeof(afs_int32));
+            temp  = rx_Read(rxcallp, (char *)&nbytes, sizeof(afs_int32));
             if (temp == sizeof(afs_int32)) {
                 nbytes = ntohl(nbytes);
                 if (nbytes > biod.length) 
-                    code = (callp->error < 0) ? callp->error : -1;
+                    code = (rxcallp->error < 0) ? rxcallp->error : -1;
             }
             else 
-                code = (callp->error < 0) ? callp->error : -1;
+                code = (rxcallp->error < 0) ? rxcallp->error : -1;
         }
 #endif
 
@@ -1604,9 +1596,9 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *userp
 
                 /* read rbytes of data */
                 rbytes = (nbytes > cm_data.buf_blockSize? cm_data.buf_blockSize : nbytes);
-                temp = rx_Read(callp, bufferp, rbytes);
+                temp = rx_Read(rxcallp, bufferp, rbytes);
                 if (temp < rbytes) {
-                    code = (callp->error < 0) ? callp->error : -1;
+                    code = (rxcallp->error < 0) ? rxcallp->error : -1;
                     break;
                 }
 
@@ -1671,9 +1663,9 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *userp
 
         if (code == 0) {
             if (SERVERHAS64BIT(connp))
-                code = EndRXAFS_FetchData64(callp, &afsStatus, &callback, &volSync);
+                code = EndRXAFS_FetchData64(rxcallp, &afsStatus, &callback, &volSync);
             else
-                code = EndRXAFS_FetchData(callp, &afsStatus, &callback, &volSync);
+                code = EndRXAFS_FetchData(rxcallp, &afsStatus, &callback, &volSync);
         } else {
             if (SERVERHAS64BIT(connp))
                 osi_Log1(afsd_logp, "CALL EndRXAFS_FetchData64 skipped due to error %d", code);
@@ -1681,8 +1673,8 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *userp
                 osi_Log1(afsd_logp, "CALL EndRXAFS_FetchData skipped due to error %d", code);
         }
 
-        if (callp)
-            code = rx_EndCall(callp, code);
+        if (rxcallp)
+            code = rx_EndCall(rxcallp, code);
 
         if (code == RXKADUNKNOWNKEY)
             osi_Log0(afsd_logp, "CALL EndCall returns RXKADUNKNOWNKEY");
@@ -1696,8 +1688,6 @@ long cm_GetBuffer(cm_scache_t *scp, cm_buf_t *bufp, int *cpffp, cm_user_t *userp
 
     lock_ObtainWrite(&scp->rw);
     
-    cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_FETCHSTATUS);
-
     /* we know that no one else has changed the buffer, since we still have
      * the fetching flag on the buffers, and we have the scp locked again.
      * Copy in the version # into the buffer if we got code 0 back from the
