@@ -16,7 +16,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_flock.c,v 1.24 2003/07/15 23:14:30 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_flock.c,v 1.29.2.6 2008/05/23 14:25:16 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -24,6 +24,11 @@ RCSID
 #include "afs/afs_cbqueue.h"
 #include "afs/nfsclient.h"
 #include "afs/afs_osidnlc.h"
+#include "afs/unified_afs.h"
+
+
+
+
 
 /* Static prototypes */
 static int HandleGetLock(register struct vcache *avc,
@@ -82,7 +87,7 @@ lockIdSet(struct AFS_FLOCK *flock, struct SimpleLocks *slp, int clid)
 #endif
 	slp->pid = clid;
 #else
-#if	defined(AFS_SUN_ENV) || defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if	defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 	slp->pid = clid;
 #else
 #if defined(AFS_LINUX20_ENV) || defined(AFS_HPUX_ENV)
@@ -121,7 +126,7 @@ lockIdSet(struct AFS_FLOCK *flock, struct SimpleLocks *slp, int clid)
 #endif
 	flock->l_pid = clid;
 #else
-#if	defined(AFS_SUN_ENV) || defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if	defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 	flock->l_pid = clid;
 #else
 #if defined(AFS_LINUX20_ENV) || defined(AFS_HPUX_ENV)
@@ -214,6 +219,9 @@ lockIdcmp2(struct AFS_FLOCK *flock1, struct vcache *vp,
     94.04.13 add "force" parameter.  If a child explicitly unlocks a
     file, I guess we'll permit it.  however, we don't want simple,
     innocent closes by children to unlock files in the parent process.
+
+    If called when disconnected support is unabled, the discon_lock must
+    be held
 */
 /* clid - nonzero on sgi sunos osf1 only */
 int
@@ -226,7 +234,8 @@ HandleFlock(register struct vcache *avc, int acom, struct vrequest *areq,
     struct AFSVolSync tsync;
     afs_int32 lockType;
     struct AFS_FLOCK flock;
-    XSTATS_DECLS AFS_STATCNT(HandleFlock);
+    XSTATS_DECLS;
+    AFS_STATCNT(HandleFlock);
     code = 0;			/* default when we don't make any network calls */
     lockIdSet(&flock, NULL, clid);
 
@@ -289,20 +298,25 @@ HandleFlock(register struct vcache *avc, int acom, struct vrequest *areq,
 	    avc->slocks = 0;
 	}
 	if (avc->flockCount == 0) {
-	    do {
-		tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
-		if (tc) {
-		    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_RELEASELOCK);
-		    RX_AFS_GUNLOCK();
-		    code = RXAFS_ReleaseLock(tc->id, (struct AFSFid *)
-					     &avc->fid.Fid, &tsync);
-		    RX_AFS_GLOCK();
-		    XSTATS_END_TIME;
-		} else
+	    if (!AFS_IS_DISCONNECTED) {
+	        do {
+		    tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
+		    if (tc) {
+		        XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_RELEASELOCK);
+		        RX_AFS_GUNLOCK();
+		        code = RXAFS_ReleaseLock(tc->id, (struct AFSFid *)
+					         &avc->fid.Fid, &tsync);
+		        RX_AFS_GLOCK();
+		        XSTATS_END_TIME;
+		    } else
 		    code = -1;
-	    } while (afs_Analyze
-		     (tc, code, &avc->fid, areq,
-		      AFS_STATS_FS_RPCIDX_RELEASELOCK, SHARED_LOCK, NULL));
+	        } while (afs_Analyze
+		         (tc, code, &avc->fid, areq,
+		          AFS_STATS_FS_RPCIDX_RELEASELOCK, SHARED_LOCK, NULL));
+	    } else {
+	  	/*printf("Network is dooooooowwwwwwwnnnnnnn\n");*/
+	       code = ENETDOWN;
+	    }
 	}
     } else {
 	while (1) {		/* set a new lock */
@@ -342,24 +356,26 @@ HandleFlock(register struct vcache *avc, int acom, struct vrequest *areq,
 		    }
 		}
 		if (!code && avc->flockCount == 0) {
-		    do {
-			tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
-			if (tc) {
-			    XSTATS_START_TIME
-				(AFS_STATS_FS_RPCIDX_RELEASELOCK);
-			    RX_AFS_GUNLOCK();
-			    code =
-				RXAFS_ReleaseLock(tc->id,
-						  (struct AFSFid *)&avc->fid.
-						  Fid, &tsync);
-			    RX_AFS_GLOCK();
-			    XSTATS_END_TIME;
-			} else
-			    code = -1;
-		    } while (afs_Analyze
-			     (tc, code, &avc->fid, areq,
-			      AFS_STATS_FS_RPCIDX_RELEASELOCK, SHARED_LOCK,
-			      NULL));
+		    if (!AFS_IS_DISCONNECTED) {
+		        do {
+			    tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
+			    if (tc) {
+			        XSTATS_START_TIME
+				    (AFS_STATS_FS_RPCIDX_RELEASELOCK);
+			        RX_AFS_GUNLOCK();
+			        code =
+				    RXAFS_ReleaseLock(tc->id,
+						      (struct AFSFid *)&avc->
+						      fid.Fid, &tsync);
+			        RX_AFS_GLOCK();
+			       XSTATS_END_TIME;
+			    } else
+			        code = -1;
+		        } while (afs_Analyze
+			         (tc, code, &avc->fid, areq,
+			          AFS_STATS_FS_RPCIDX_RELEASELOCK, SHARED_LOCK,
+			          NULL));
+		    }
 		}
 	    } else if (avc->flockCount == -1 && (acom & LOCK_EX)) {
 		if (lockIdcmp2(&flock, avc, NULL, 1, clid)) {
@@ -375,22 +391,28 @@ HandleFlock(register struct vcache *avc, int acom, struct vrequest *areq,
 		if (avc->flockCount == 0) {
 		    /* we're the first on our block, send the call through */
 		    lockType = ((acom & LOCK_EX) ? LockWrite : LockRead);
-		    do {
-			tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
-			if (tc) {
-			    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_SETLOCK);
-			    RX_AFS_GUNLOCK();
-			    code = RXAFS_SetLock(tc->id, (struct AFSFid *)
-						 &avc->fid.Fid, lockType,
-						 &tsync);
-			    RX_AFS_GLOCK();
-			    XSTATS_END_TIME;
-			} else
-			    code = -1;
-		    } while (afs_Analyze
-			     (tc, code, &avc->fid, areq,
-			      AFS_STATS_FS_RPCIDX_SETLOCK, SHARED_LOCK,
-			      NULL));
+		    if (!AFS_IS_DISCONNECTED) {
+		        do {
+			    tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
+			    if (tc) {
+			        XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_SETLOCK);
+			        RX_AFS_GUNLOCK();
+			        code = RXAFS_SetLock(tc->id, (struct AFSFid *)
+						     &avc->fid.Fid, lockType,
+						     &tsync);
+			        RX_AFS_GLOCK();
+			        XSTATS_END_TIME;
+			    } else
+			        code = -1;
+		        } while (afs_Analyze
+			         (tc, code, &avc->fid, areq,
+			          AFS_STATS_FS_RPCIDX_SETLOCK, SHARED_LOCK,
+			          NULL));
+		    } else
+		        /* XXX - Should probably try and log this when we're
+		         * XXX - running with logging enabled. But it's horrid
+		         */
+		        code = 0; /* pretend we worked - ick!!! */
 		} else
 		    code = 0;	/* otherwise, pretend things worked */
 	    }
@@ -421,7 +443,8 @@ HandleFlock(register struct vcache *avc, int acom, struct vrequest *areq,
 		break;
 	    }
 	    /* now, if we got EWOULDBLOCK, and we're supposed to wait, we do */
-	    if (((code == EWOULDBLOCK) || (code == EAGAIN))
+	    if (((code == EWOULDBLOCK) || (code == EAGAIN) || 
+		 (code == UAEWOULDBLOCK) || (code == UAEAGAIN))
 		&& !(acom & LOCK_NB)) {
 		/* sleep for a second, allowing interrupts */
 		ReleaseWriteLock(&avc->lock);
@@ -468,15 +491,15 @@ DoLockWarning(void)
 
 
 #ifdef	AFS_OSF_ENV
-afs_lockctl(struct vcache * avc, struct eflock * af, int flag,
-	    struct AFS_UCRED * acred, pid_t clid, off_t offset)
-#elif defined(AFS_SGI_ENV) || (defined(AFS_SUN_ENV) && !defined(AFS_SUN5_ENV)) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
-afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
-	    struct AFS_UCRED * acred, pid_t clid)
+int afs_lockctl(struct vcache * avc, struct eflock * af, int flag,
+		struct AFS_UCRED * acred, pid_t clid, off_t offset)
+#elif defined(AFS_SGI_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+int afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
+		struct AFS_UCRED * acred, pid_t clid)
 #else
 u_int clid = 0;
-afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
-	    struct AFS_UCRED * acred)
+int afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
+		struct AFS_UCRED * acred)
 #endif
 {
     struct vrequest treq;
@@ -490,15 +513,17 @@ afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
     if ((code = afs_InitReq(&treq, acred)))
 	return code;
     afs_InitFakeStat(&fakestate);
+
+    AFS_DISCON_LOCK();
+
     code = afs_EvalFakeStat(&avc, &fakestate, &treq);
     if (code) {
-	afs_PutFakeStat(&fakestate);
-	return code;
+	goto done;
     }
 #ifdef	AFS_OSF_ENV
     if (flag & VNOFLCK) {
-	afs_PutFakeStat(&fakestate);
-	return 0;
+	code = 0;
+	goto done;
     }
     if (flag & CLNFLCK) {
 	acmd = LOCK_UN;
@@ -508,23 +533,22 @@ afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
 	acmd = F_SETLK;
     }
 #endif
-#if (defined(AFS_SUN_ENV) || defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV)) && !defined(AFS_SUN58_ENV)
+#if (defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV)) && !defined(AFS_SUN58_ENV)
     if ((acmd == F_GETLK) || (acmd == F_RGETLK)) {
 #else
     if (acmd == F_GETLK) {
 #endif
 	if (af->l_type == F_UNLCK) {
-	    afs_PutFakeStat(&fakestate);
-	    return 0;
+	    code = 0;
+	    goto done;
 	}
 #ifndef	AFS_OSF_ENV		/* getlock is a no-op for osf (for now) */
 	code = HandleGetLock(avc, af, &treq, clid);
 #endif
 	code = afs_CheckCode(code, &treq, 2);	/* defeat buggy AIX optimz */
-	afs_PutFakeStat(&fakestate);
-	return code;
+	goto done;
     } else if ((acmd == F_SETLK) || (acmd == F_SETLKW)
-#if (defined(AFS_SUN_ENV) || defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV)) && !defined(AFS_SUN58_ENV)
+#if (defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV)) && !defined(AFS_SUN58_ENV)
 	       || (acmd == F_RSETLK) || (acmd == F_RSETLKW)) {
 #else
 	) {
@@ -544,12 +568,16 @@ afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
 	af->l_len = 0;		/* since some systems indicate it as EOF */
 #endif
 #endif
+    /* Java VMs ask for l_len=(long)-1 regardless of OS/CPU; bottom 32 bits
+     * sometimes get masked off by OS */
+    if ((sizeof(af->l_len) == 8) && (af->l_len == 0x7ffffffffffffffe))
+	af->l_len = 0;
     /* next line makes byte range locks always succeed,
      * even when they should block */
     if (af->l_whence != 0 || af->l_start != 0 || af->l_len != 0) {
 	DoLockWarning();
-	afs_PutFakeStat(&fakestate);
-	return 0;
+	code = 0;
+	goto done;
     }
     /* otherwise we can turn this into a whole-file flock */
     if (af->l_type == F_RDLCK)
@@ -563,12 +591,12 @@ afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
 	return EINVAL;		/* unknown lock type */
     }
     if (((acmd == F_SETLK)
-#if 	(defined(AFS_SGI_ENV) || defined(AFS_SUN_ENV)) && !defined(AFS_SUN58_ENV)
+#if 	(defined(AFS_SGI_ENV) || defined(AFS_SUN5_ENV)) && !defined(AFS_SUN58_ENV)
 	 || (acmd == F_RSETLK)
 #endif
 	) && code != LOCK_UN)
 	code |= LOCK_NB;	/* non-blocking, s.v.p. */
-#if	(defined(AFS_SUN_ENV) && !defined(AFS_SUN5_ENV)) || defined(AFS_OSF_ENV)
+#if	defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV)
     code = HandleFlock(avc, code, &treq, clid, 0 /*!onlymine */ );
 #elif defined(AFS_SGI_ENV)
     AFS_RWLOCK((vnode_t *) avc, VRWLOCK_WRITE);
@@ -578,11 +606,13 @@ afs_lockctl(struct vcache * avc, struct AFS_FLOCK * af, int acmd,
     code = HandleFlock(avc, code, &treq, 0, 0 /*!onlymine */ );
 #endif
     code = afs_CheckCode(code, &treq, 3);	/* defeat AIX -O bug */
-    afs_PutFakeStat(&fakestate);
-    return code;
+    goto done;
     }
+    code = EINVAL;
+done:
     afs_PutFakeStat(&fakestate);
-    return EINVAL;
+    AFS_DISCON_UNLOCK();
+    return code;
 }
 
 
@@ -804,9 +834,14 @@ GetFlockCount(struct vcache *avc, struct vrequest *areq)
     struct AFSCallBack CallBack;
     struct AFSVolSync tsync;
     int temp;
-    XSTATS_DECLS temp = areq->flags & O_NONBLOCK;
+    XSTATS_DECLS;
+    temp = areq->flags & O_NONBLOCK;
     areq->flags |= O_NONBLOCK;
 
+    /* If we're disconnected, lie and say that we've got no locks. Ick */
+    if (AFS_IS_DISCONNECTED)
+        return 0;
+        
     do {
 	tc = afs_Conn(&avc->fid, areq, SHARED_LOCK);
 	if (tc) {
@@ -877,31 +912,24 @@ afs_xflock(void)
 	afs_PutFakeStat(&fakestate);
 	return flockDone;
     }
+
+    AFS_DISCON_LOCK();
+    
     /* first determine whether this is any sort of vnode */
     if (fd->f_type == DTYPE_VNODE) {
 	/* good, this is a vnode; next see if it is an AFS vnode */
 	tvc = VTOAFS(fd->f_data);	/* valid, given a vnode */
 	if (IsAfsVnode(AFSTOV(tvc))) {
 	    /* This is an AFS vnode, so do the work */
-#ifdef AFS_DEC_ENV
-	    /* find real vcache entry; shouldn't be null if gnode ref count
-	     * is greater than 0.
-	     */
-	    tvc = VTOAFS(afs_gntovn) (tvc);
-	    if (!tvc) {
-		u.u_error = ENOENT;
-		afs_PutFakeStat(&fakestate);
-		return;
-	    }
-#endif
 	    code = afs_EvalFakeStat(&tvc, &fakestate, &treq);
 	    if (code) {
+		AFS_DISCON_UNLOCK();
 		afs_PutFakeStat(&fakestate);
 		return code;
 	    }
 	    if ((fd->f_flag & (FEXLOCK | FSHLOCK)) && !(uap->com & LOCK_UN)) {
 		/* First, if fd already has lock, release it for relock path */
-#if defined(AFS_SGI_ENV) || defined(AFS_OSF_ENV) || (defined(AFS_SUN_ENV) && !defined(AFS_SUN5_ENV))
+#if defined(AFS_SGI_ENV) || defined(AFS_OSF_ENV)
 		HandleFlock(tvc, LOCK_UN, &treq, u.u_procp->p_pid,
 			    0 /*!onlymine */ );
 #else
@@ -911,7 +939,7 @@ afs_xflock(void)
 	    }
 	    /* now try the requested operation */
 
-#if defined(AFS_SGI_ENV) || defined(AFS_OSF_ENV) || (defined(AFS_SUN_ENV) && !defined(AFS_SUN5_ENV))
+#if defined(AFS_SGI_ENV) || defined(AFS_OSF_ENV)
 	    code =
 		HandleFlock(tvc, uap->com, &treq, u.u_procp->p_pid,
 			    0 /*!onlymine */ );
@@ -949,6 +977,7 @@ afs_xflock(void)
 #else
     FP_UNREF(fd);
 #endif
+    AFS_DISCON_UNLOCK();
     afs_PutFakeStat(&fakestate);
     return code;
 #else /* AFS_OSF_ENV */
@@ -958,6 +987,7 @@ afs_xflock(void)
 #else
 	flock();
 #endif
+    AFS_DISCON_UNLOCK();
     afs_PutFakeStat(&fakestate);
     return;
 #endif
