@@ -10,7 +10,7 @@
 #ifndef __AUTH_AFS_INCL_
 #define	__AUTH_AFS_INCL_    1
 
-#include <rx/rxkad.h>			/* to get ticket parameters/contents */
+#include <rx/rxkad.h>		/* to get ticket parameters/contents */
 
 /* super-user pincipal used by servers when talking to other servers */
 #define AUTH_SUPERUSER        "afs"
@@ -19,37 +19,20 @@ struct ktc_token {
     afs_int32 startTime;
     afs_int32 endTime;
     struct ktc_encryptionKey sessionKey;
-    short kvno;  /* XXX UNALIGNED */
+    short kvno;			/* XXX UNALIGNED */
     int ticketLen;
     char ticket[MAXKTCTICKETLEN];
 };
 
+int ktc_SetToken(struct ktc_principal *, struct ktc_token *,
+    struct ktc_principal *, afs_int32);
+int ktc_GetToken(struct ktc_principal *, struct ktc_token *,
+    int, struct ktc_principal *);
+int ktc_ListTokens(int, int *, struct ktc_principal *);
+int ktc_ForgetToken(struct ktc_principal *);
+int ktc_ForgetAllTokens(void);
+
 #ifdef AFS_NT40_ENV
-extern int ktc_SetToken(
-        struct ktc_principal *server,
-        struct ktc_token *token,
-        struct ktc_principal *client,
-        afs_int32 flags
-);
-
-extern int ktc_GetToken(
-        struct ktc_principal *server,
-        struct ktc_token *token,
-        int tokenLen,
-        struct ktc_principal *client
-);
-
-extern int ktc_ListTokens(
-        int cellNum,
-        int *cellNumP,
-        struct ktc_principal *serverName
-);
-
-extern int ktc_ForgetToken(
-        struct ktc_principal *server
-);
-
-extern int ktc_ForgetAllTokens(void);
 
 /* Flags for the flag word sent along with a token */
 #define PIOCTL_LOGON		0x1	/* invoked from integrated logon */
