@@ -8,16 +8,17 @@
  */
 
 #include <afsconfig.h>
-#include "../afs/param.h"
+#include "afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/rx/UKERNEL/rx_knet.c,v 1.4 2001/07/12 19:59:01 shadow Exp $");
+RCSID
+    ("$Header: /cvs/openafs/src/rx/UKERNEL/rx_knet.c,v 1.13.4.1 2008/03/10 22:32:34 shadow Exp $");
 
-#include "../rx/rx_kcommon.h"
+#include "rx/rx_kcommon.h"
 
 
 #define SECONDS_TO_SLEEP	0
-#define NANO_SECONDS_TO_SLEEP	100000000 /* 100 milliseconds */
-#define LOOPS_PER_WAITCHECK	10	  /* once per second */
+#define NANO_SECONDS_TO_SLEEP	100000000	/* 100 milliseconds */
+#define LOOPS_PER_WAITCHECK	10	/* once per second */
 
 unsigned short usr_rx_port = 0;
 
@@ -26,23 +27,23 @@ struct usr_in_ifaddr *usr_in_ifaddr = NULL;
 
 void rxk_InitializeSocket();
 
-void afs_rxevent_daemon(argp)
-void *argp;
+void
+afs_rxevent_daemon(void)
 {
     struct timespec tv;
     struct clock temp;
     int i = 0;
 
     AFS_GUNLOCK();
-    while(1) {
+    while (1) {
 	tv.tv_sec = SECONDS_TO_SLEEP;
 	tv.tv_nsec = NANO_SECONDS_TO_SLEEP;
 	usr_thread_sleep(&tv);
 	/*
 	 * Check for shutdown, don't try to stop the listener
 	 */
-        if (afs_termState == AFSOP_STOP_RXEVENT ||
-	    afs_termState == AFSOP_STOP_RXK_LISTENER) {
+	if (afs_termState == AFSOP_STOP_RXEVENT
+	    || afs_termState == AFSOP_STOP_RXK_LISTENER) {
 	    AFS_GLOCK();
 	    afs_termState = AFSOP_STOP_COMPLETE;
 	    afs_osi_Wakeup(&afs_termState);
@@ -59,10 +60,8 @@ void *argp;
 
 /* Loop to listen on a socket. Return setting *newcallp if this
  * thread should become a server thread.  */
-void rxi_ListenerProc(usockp, tnop, newcallp)
-osi_socket usockp;
-int *tnop;
-struct rx_call **newcallp;
+void
+rxi_ListenerProc(osi_socket usockp, int *tnop, struct rx_call **newcallp)
 {
     struct rx_packet *tp;
     afs_uint32 host;
@@ -77,7 +76,7 @@ struct rx_call **newcallp;
     while (1) {
 	tp = rxi_AllocPacket(RX_PACKET_CLASS_RECEIVE);
 	usr_assert(tp != NULL);
-	rc = rxi_ReadPacket(usockp->sock, tp, &host, &port);
+	rc = rxi_ReadPacket(usockp, tp, &host, &port);
 	if (rc != 0) {
 	    tp = rxi_ReceivePacket(tp, usockp, host, port, tnop, newcallp);
 	    if (newcallp && *newcallp) {
@@ -90,7 +89,7 @@ struct rx_call **newcallp;
 	if (tp) {
 	    rxi_FreePacket(tp);
 	}
-	if (afs_termState == AFSOP_STOP_RXEVENT ) {
+	if (afs_termState == AFSOP_STOP_RXEVENT) {
 	    afs_termState = AFSOP_STOP_RXK_LISTENER;
 	    afs_osi_Wakeup(&afs_termState);
 	}
@@ -100,7 +99,8 @@ struct rx_call **newcallp;
 /* This is the listener process request loop. The listener process loop
  * becomes a server thread when rxi_ListenerProc returns, and stays
  * server thread until rxi_ServerProc returns. */
-void rxk_Listener(void)
+void
+rxk_Listener(void)
 {
     int threadID;
     osi_socket sock = (osi_socket) rx_socket;
@@ -116,7 +116,7 @@ void rxk_Listener(void)
     assert(usockp != NULL);
 
     AFS_GUNLOCK();
-    while(1) {
+    while (1) {
 	newcall = NULL;
 	threadID = -1;
 	rxi_ListenerProc(sock, &threadID, &newcall);
@@ -134,20 +134,21 @@ void rxk_Listener(void)
 /* This is the server process request loop. The server process loop
  * becomes a listener thread when rxi_ServerProc returns, and stays
  * listener thread until rxi_ListenerProc returns. */
-void rx_ServerProc()
+void *
+rx_ServerProc(void *unused)
 {
     osi_socket sock;
     int threadID;
     struct rx_call *newcall = NULL;
 
-    rxi_MorePackets(rx_maxReceiveWindow+2); /* alloc more packets */
+    rxi_MorePackets(rx_maxReceiveWindow + 2);	/* alloc more packets */
     rxi_dataQuota += rx_initSendWindow;	/* Reserve some pkts for hard times */
     /* threadID is used for making decisions in GetCall.  Get it by bumping
      * number of threads handling incoming calls */
     threadID = rxi_availProcs++;
 
     AFS_GUNLOCK();
-    while(1) {
+    while (1) {
 	sock = OSI_NULLSOCKET;
 	rxi_ServerProc(threadID, newcall, &sock);
 	if (sock == OSI_NULLSOCKET) {
@@ -168,7 +169,8 @@ void rx_ServerProc()
  * routines. Allocate the socket buffer here, but don't open it until
  * we start the receiver threads.
  */
-struct osi_socket *rxk_NewSocket(short aport)
+osi_socket *
+rxk_NewSocketHost(afs_uint32 ahost, short aport)
 {
     struct usr_socket *usockp;
 
@@ -177,7 +179,13 @@ struct osi_socket *rxk_NewSocket(short aport)
 
     usockp->sock = -1;
 
-    return (struct osi_socket *)usockp;
+    return (osi_socket *)usockp;
+}
+
+osi_socket *
+rxk_NewSocket(short aport)
+{
+    return rxk_NewSocketHost(htonl(INADDR_ANY), aport);
 }
 
 /*
@@ -186,7 +194,8 @@ struct osi_socket *rxk_NewSocket(short aport)
  * we allocated in rxk_NewSocket. Now is the time to bind our
  * socket and start the receiver threads.
  */
-void rxk_InitializeSocket()
+void
+rxk_InitializeSocket(void)
 {
     int rc, sock, i;
 #ifdef AFS_USR_AIX_ENV
@@ -219,33 +228,31 @@ void rxk_InitializeSocket()
     optval0 = 131072;
 #endif
     optval = optval0;
-    rc = setsockopt(sock, SOL_SOCKET, SO_SNDBUF,
-		    (void *)&optval, sizeof(optval));
+    rc = setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (void *)&optval,
+		    sizeof(optval));
     usr_assert(rc == 0);
     optlen = sizeof(optval);
-    rc = getsockopt(sock, SOL_SOCKET, SO_SNDBUF,
-		    (void *)&optval, &optlen);
+    rc = getsockopt(sock, SOL_SOCKET, SO_SNDBUF, (void *)&optval, &optlen);
     usr_assert(rc == 0);
-    usr_assert(optval == optval0);
+    /* usr_assert(optval == optval0); */
 #ifdef AFS_USR_LINUX22_ENV
     optval0 = 131070;
 #else
     optval0 = 131072;
 #endif
     optval = optval0;
-    rc = setsockopt(sock, SOL_SOCKET, SO_RCVBUF,
-		    (void *)&optval, sizeof(optval));
+    rc = setsockopt(sock, SOL_SOCKET, SO_RCVBUF, (void *)&optval,
+		    sizeof(optval));
     usr_assert(rc == 0);
     optlen = sizeof(optval);
-    rc = getsockopt(sock, SOL_SOCKET, SO_RCVBUF,
-		    (void *)&optval, &optlen);
+    rc = getsockopt(sock, SOL_SOCKET, SO_RCVBUF, (void *)&optval, &optlen);
     usr_assert(rc == 0);
-    usr_assert(optval == optval0);
+    /* usr_assert(optval == optval0); */
 
 #ifdef AFS_USR_AIX_ENV
     optval = 1;
-    rc = setsockopt(sock, SOL_SOCKET, SO_CKSUMRECV,
-		    (void *)&optval, sizeof(optval));
+    rc = setsockopt(sock, SOL_SOCKET, SO_CKSUMRECV, (void *)&optval,
+		    sizeof(optval));
     usr_assert(rc == 0);
 #endif /* AFS_USR_AIX_ENV */
 
@@ -259,24 +266,21 @@ void rxk_InitializeSocket()
     rx_port = usockp->port;
 }
 
-int rxk_FreeSocket(sockp)
-struct usr_socket *sockp;
+int
+rxk_FreeSocket(struct usr_socket *sockp)
 {
     return 0;
 }
 
-void osi_StopListener(void)
+void
+osi_StopListener(void)
 {
     rxk_FreeSocket((struct usr_socket *)rx_socket);
 }
 
-int osi_NetSend(sockp, addr, iov, nio, size, stack) 
-    struct osi_socket *sockp;
-    struct sockaddr_in *addr; 
-    struct iovec *iov;
-    int nio;
-    afs_int32 size;
-    int stack;
+int
+osi_NetSend(osi_socket sockp, struct sockaddr_in *addr, struct iovec *iov,
+	    int nio, afs_int32 size, int stack)
 {
     int rc;
     int i;
@@ -289,7 +293,7 @@ int osi_NetSend(sockp, addr, iov, nio, size, stack)
      * The header is in the first iovec
      */
     usr_assert(nio > 0 && nio <= 64);
-    for (i = 0 ; i < nio ; i++) {
+    for (i = 0; i < nio; i++) {
 	tmpiov[i].iov_base = iov[i].iov_base;
 	tmpiov[i].iov_len = iov[i].iov_len;
     }
@@ -309,13 +313,15 @@ int osi_NetSend(sockp, addr, iov, nio, size, stack)
     return 0;
 }
 
-void shutdown_rxkernel(void)
+void
+shutdown_rxkernel(void)
 {
     rxk_initDone = 0;
     rxk_shutdownPorts();
 }
 
-void rx_Finalize()
+void
+rx_Finalize(void)
 {
     usr_assert(0);
 }
@@ -324,15 +330,12 @@ void rx_Finalize()
  * Recvmsg.
  *
  */
-int rxi_Recvmsg
-    (int socket,
-     struct msghdr *msg_p,
-     int flags)
+int
+rxi_Recvmsg(osi_socket socket, struct msghdr *msg_p, int flags)
 {
     int ret;
     do {
-	ret = recvmsg(socket, msg_p, flags);
+	ret = recvmsg(socket->sock, msg_p, flags);
     } while (ret == -1 && errno == EAGAIN);
     return ret;
 }
-
