@@ -15,7 +15,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_kcommon.c,v 1.56.2.7 2008/03/10 22:32:34 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_kcommon.c,v 1.56.2.10 2008/10/03 14:55:12 jaltman Exp $");
 
 #include "rx/rx_kcommon.h"
 
@@ -126,13 +126,14 @@ rxi_GetUDPSocket(u_short port)
 
 #if !defined(AFS_LINUX26_ENV)
 void
-osi_Panic(msg, a1, a2, a3)
-     char *msg;
+osi_Panic(char *msg, ...)
 {
+    va_list ap;
     if (!msg)
 	msg = "Unknown AFS panic";
 
-    printf(msg, a1, a2, a3);
+    va_start(ap, msg);
+    vprintf(msg, ap);
 #ifdef AFS_LINUX20_ENV
     * ((char *) 0) = 0; 
 #else
@@ -291,6 +292,8 @@ rx_ServerProc(void *unused)
 #ifdef RX_ENABLE_LOCKS
     AFS_GLOCK();
 #endif /* RX_ENABLE_LOCKS */
+
+    return NULL;
 }
 #endif /* !UKERNEL */
 
@@ -473,7 +476,7 @@ rxi_InitPeerParams(register struct rx_peer *pp)
     pp->natMTU = MIN(pp->ifMTU, OLD_MAX_PACKET_SIZE);
     pp->ifDgramPackets =
 	MIN(rxi_nDgramPackets,
-	    rxi_AdjustDgramPackets(RX_MAX_FRAGS, pp->ifMTU));
+	    rxi_AdjustDgramPackets(rxi_nSendFrags, pp->ifMTU));
     pp->maxDgramPackets = 1;
 
     /* Initialize slow start parameters */
