@@ -11,7 +11,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/ubik/phys.c,v 1.9.2.1 2007/10/30 15:16:47 shadow Exp $");
+    ("$Header: /cvs/openafs/src/ubik/phys.c,v 1.9.2.6 2008/10/07 17:44:13 shadow Exp $");
 
 #include <sys/types.h>
 #ifdef AFS_NT40_ENV
@@ -23,7 +23,14 @@ RCSID
 #include <netinet/in.h>
 #endif
 #include <sys/stat.h>
+
+/* #ifdef AFS_PTHREAD_ENV */
+#if 0   /* temporary hack - klm */
+/* nothing */
+#else
 #include <lwp.h>
+#endif
+
 #include <lock.h>
 #include <errno.h>
 #include <string.h>
@@ -78,15 +85,8 @@ uphys_open(register struct ubik_dbase *adbase, afs_int32 afid)
     }
 
     /* not found, open it and try to enter in cache */
-    strcpy(pbuffer, adbase->pathName);
-    strcat(pbuffer, ".DB");
-    if (afid < 0) {
-	i = -afid;
-	strcat(pbuffer, "SYS");
-    } else
-	i = afid;
-    sprintf(temp, "%d", i);
-    strcat(pbuffer, temp);
+    afs_snprintf(pbuffer, sizeof(pbuffer), "%s.DB%s%d", adbase->pathName, 
+		 (afid<0)?"SYS":"", (afid<0)?-afid:afid);
     fd = open(pbuffer, O_CREAT | O_RDWR, 0600);
     if (fd < 0) {
 	/* try opening read-only */
@@ -115,7 +115,7 @@ uphys_open(register struct ubik_dbase *adbase, afs_int32 afid)
     }
     if (bestfd) {		/* found a usable slot */
 	tfd = bestfd;
-	if (tfd->fd >= 0)
+	if (tfd->fd >= 0) 
 	    close(tfd->fd);
 	tfd->fd = fd;
 	tfd->refCount = 1;	/* us */
@@ -137,10 +137,22 @@ uphys_close(register int afd)
 	return EBADF;
     tfd = fdcache;
     for (i = 0; i < MAXFDCACHE; i++, tfd++) {
-	if (tfd->fd == afd) {
-	    tfd->refCount--;
-	    return 0;
-	}
+	if (tfd->fd == afd) 
+	    if (tfd->fileID != -10000) {
+		tfd->refCount--;
+		return 0;
+	    } else {
+		if (tfd->refCount > 0) {
+		    tfd->refCount--;
+		    if (tfd->refCount == 0) {
+			close(tfd->fd);
+			tfd->fd = -1;
+		    }
+		    return 0;
+		}
+		tfd->fd = -1;
+		break;
+	    }
     }
     return close(afd);
 }
@@ -289,3 +301,22 @@ uphys_sync(register struct ubik_dbase *adbase, afs_int32 afile)
     uphys_close(fd);
     return code;
 }
+
+void
+uphys_invalidate(register struct ubik_dbase *adbase, afs_int32 afid)
+{
+    register int i;
+    register struct fdcache *tfd;
+
+    /* scan file descr cache */
+    for (tfd = fdcache, i = 0; i < MAXFDCACHE; i++, tfd++) {
+	if (afid == tfd->fileID) {
+	    tfd->fileID = -10000;
+	    if (tfd->fd >= 0 && tfd->refCount == 0) {
+		close(tfd->fd);
+		tfd->fd = -1;
+	    }
+	    return;
+	}
+    }
+}
