@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/venus/dedebug.c,v 1.1.4.1 2007/02/01 13:25:33 jaltman Exp $");
+RCSID("$Header: /cvs/openafs/src/venus/dedebug.c,v 1.1.4.3 2007/10/31 04:09:39 shadow Exp $");
 
 
 #include <sys/types.h>
@@ -41,7 +41,7 @@ static PrintCacheEntries(struct rx_connection *aconn, int aint32)
 	if (code) {
 	    if (code == 1) break;
 	    printf("cmdebug: failed to get cache entry %d (%s)\n", i,
-		   error_message(code));
+		   afs_error_message(code));
 	    return code;
 	}
 
@@ -55,8 +55,9 @@ static PrintCacheEntries(struct rx_connection *aconn, int aint32)
     return 0;
 }
 
-static CommandProc(as)
-struct cmd_syndesc *as; {
+static int
+CommandProc(struct cmd_syndesc *as, void *arock)
+{
     struct rx_connection *conn;
     register char *hostName;
     register struct hostent *thp;
@@ -111,7 +112,7 @@ char **argv; {
 #endif
     rx_Init(0);
 
-    ts = cmd_CreateSyntax((char *) 0, CommandProc, 0, "probe unik server");
+    ts = cmd_CreateSyntax(NULL, CommandProc, NULL, "probe unik server");
     cmd_AddParm(ts, "-servers", CMD_SINGLE, CMD_REQUIRED, "server machine");
     cmd_AddParm(ts, "-port", CMD_SINGLE, CMD_OPTIONAL, "IP port");
     cmd_AddParm(ts, "-long", CMD_FLAG, CMD_OPTIONAL, "print all info");
