@@ -22,7 +22,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/viced/viced.c,v 1.75.2.23 2008/05/09 18:50:57 shadow Exp $");
+    ("$Header: /cvs/openafs/src/viced/viced.c,v 1.75.2.25 2008/09/24 21:36:49 shadow Exp $");
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -162,7 +162,7 @@ int SawSpare;
 int SawPctSpare;
 int debuglevel = 0;
 int printBanner = 0;
-int rxJumbograms = 1;		/* default is to send and receive jumbograms. */
+int rxJumbograms = 0;		/* default is to not send and receive jumbograms. */
 int rxBind = 0;		/* don't bind */
 int rxkadDisableDotCheck = 0;      /* disable check for dot in principal name */ 
 int rxMaxMTU = -1;
@@ -256,7 +256,8 @@ CheckDescriptors(void *unused)
     for (i = 0; i < tsize; i++) {
 	if (afs_fstat(i, &status) != -1) {
 	    printf("%d: dev %x, inode %u, length %u, type/mode %x\n", i,
-		   status.st_dev, status.st_ino, status.st_size,
+		   status.st_dev, status.st_ino, 
+		   (unsigned int) status.st_size,
 		   status.st_mode);
 	}
     }
@@ -921,6 +922,8 @@ FlagMsg()
     strcat(buffer, "[-udpsize <size of socket buffer in bytes>] ");
     strcat(buffer, "[-sendsize <size of send buffer in bytes>] ");
     strcat(buffer, "[-abortthreshold <abort threshold>] ");
+    strcat(buffer, "[-nojumbo (disable jumbogram network packets - deprecated)] ");
+    strcat(buffer, "[-jumbo (enable jumbogram network packets)] ");
 /*   strcat(buffer, "[-enable_peer_stats] "); */
 /*   strcat(buffer, "[-enable_process_stats] "); */
     strcat(buffer, "[-help]\n");
@@ -1272,6 +1275,8 @@ ParseArgs(int argc, char *argv[])
 #endif
 	else if (!strcmp(argv[i], "-nojumbo")) {
 	    rxJumbograms = 0;
+	} else if (!strcmp(argv[i], "-jumbo")) {
+	    rxJumbograms = 1;
 	} else if (!strcmp(argv[i], "-rxbind")) {
 	    rxBind = 1;
 	} else if (!strcmp(argv[i], "-allow-dotted-principals")) {
@@ -2091,8 +2096,7 @@ main(int argc, char *argv[])
     }
     if (rxkadDisableDotCheck) {
         rx_SetSecurityConfiguration(tservice, RXS_CONFIG_FLAGS,
-                                    (void *)RXS_CONFIG_FLAGS_DISABLE_DOTCHECK,
-                                    NULL);
+                                    (void *)RXS_CONFIG_FLAGS_DISABLE_DOTCHECK);
     }
     rx_SetMinProcs(tservice, 3);
     rx_SetMaxProcs(tservice, lwps);
