@@ -19,7 +19,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/vol/fssync-debug.c,v 1.1.4.2 2008/02/04 18:51:39 shadow Exp $");
+    ("$Header: /cvs/openafs/src/vol/fssync-debug.c,v 1.1.4.5 2008/06/12 19:18:49 shadow Exp $");
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -98,6 +98,9 @@ static int VolDetach(struct cmd_syndesc * as, void * rock);
 static int VolBreakCBKs(struct cmd_syndesc * as, void * rock);
 static int VolMove(struct cmd_syndesc * as, void * rock);
 static int VolList(struct cmd_syndesc * as, void * rock);
+static int VolLeaveOff(struct cmd_syndesc * as, void * rock);
+static int VolForceAttach(struct cmd_syndesc * as, void * rock);
+static int VolForceError(struct cmd_syndesc * as, void * rock);
 static int VolQuery(struct cmd_syndesc * as, void * rock);
 static int VolHdrQuery(struct cmd_syndesc * as, void * rock);
 static int VolOpQuery(struct cmd_syndesc * as, void * rock);
@@ -105,7 +108,7 @@ static int StatsQuery(struct cmd_syndesc * as, void * rock);
 static int VnQuery(struct cmd_syndesc * as, void * rock);
 
 static void print_vol_stats_general(VolPkgStats * stats);
-static void print_vol_stats_viceP(struct DiskPartitionStats * stats);
+static void print_vol_stats_viceP(struct DiskPartitionStats64 * stats);
 static void print_vol_stats_hash(struct VolumeHashChainStats * stats);
 #ifdef AFS_DEMAND_ATTACH_FS
 static void print_vol_stats_hdr(struct volume_hdr_LRU_stats * stats);
@@ -180,6 +183,15 @@ main(int argc, char **argv)
     VOLOP_PARMS_DECL(ts);
     cmd_CreateAlias(ts, "ls");
 
+    ts = cmd_CreateSyntax("leaveoff", VolLeaveOff, 0, "leave volume offline (FSYNC_VOL_LEAVE_OFF opcode)");
+    VOLOP_PARMS_DECL(ts);
+
+    ts = cmd_CreateSyntax("attach", VolForceAttach, 0, "force full attachment (FSYNC_VOL_ATTACH opcode)");
+    VOLOP_PARMS_DECL(ts);
+
+    ts = cmd_CreateSyntax("error", VolForceError, 0, "force into hard error state (FSYNC_VOL_FORCE_ERROR opcode)");
+    VOLOP_PARMS_DECL(ts);
+
     ts = cmd_CreateSyntax("query", VolQuery, NULL, "get volume structure (FSYNC_VOL_QUERY opcode)");
     VOLOP_PARMS_DECL(ts);
     cmd_CreateAlias(ts, "qry");
@@ -500,6 +512,45 @@ VolList(struct cmd_syndesc * as, void * rock)
     return 0;
 }
 
+static int
+VolLeaveOff(struct cmd_syndesc * as, void * rock)
+{
+    struct state state;
+
+    common_prolog(as, &state);
+    common_volop_prolog(as, &state);
+
+    do_volop(&state, FSYNC_VOL_LEAVE_OFF, NULL);
+
+    return 0;
+}
+
+static int
+VolForceAttach(struct cmd_syndesc * as, void * rock)
+{
+    struct state state;
+
+    common_prolog(as, &state);
+    common_volop_prolog(as, &state);
+
+    do_volop(&state, FSYNC_VOL_ATTACH, NULL);
+
+    return 0;
+}
+
+static int
+VolForceError(struct cmd_syndesc * as, void * rock)
+{
+    struct state state;
+
+    common_prolog(as, &state);
+    common_volop_prolog(as, &state);
+
+    do_volop(&state, FSYNC_VOL_FORCE_ERROR, NULL);
+
+    return 0;
+}
+
 #ifdef AFS_DEMAND_ATTACH_FS
 static char *
 vol_state_to_string(VolState state)
@@ -759,7 +810,11 @@ VolHdrQuery(struct cmd_syndesc * as, void * rock)
 	
 	printf("\tid               = %u\n", v.id);
 	printf("\tname             = '%s'\n", v.name);
-	printf("\tinUse            = %d\n", v.inUse);
+	if (v.inUse != 0) {
+	    printf("\tinUse            = %d (%s)\n", v.inUse, program_type_to_string(v.inUse));
+	} else {
+	    printf("\tinUse            = %d (no)\n", v.inUse);
+	}
 	printf("\tinService        = %d\n", v.inService);
 	printf("\tblessed          = %d\n", v.blessed);
 	printf("\tneedsSalvaged    = %d\n", v.needsSalvaged);
@@ -831,8 +886,12 @@ VolOpQuery(struct cmd_syndesc * as, void * rock)
 
 	printf("\tcom = {\n");
 	printf("\t\tproto_version  = %u\n", vop.com.proto_version);
+	printf("\t\tpkt_seq        = %u\n", vop.com.pkt_seq);
+	printf("\t\tcom_seq        = %u\n", vop.com.com_seq);
 	printf("\t\tprogramType    = %d (%s)\n", 
 	       vop.com.programType, program_type_to_string(vop.com.programType));
+	printf("\t\tpid            = %d\n", vop.com.pid);
+	printf("\t\ttid            = %d\n", vop.com.tid);
 	printf("\t\tcommand        = %d (%s)\n", 
 	       vop.com.command, command_code_to_string(vop.com.command));
 	printf("\t\treason         = %d (%s)\n", 
@@ -1045,7 +1104,7 @@ StatsQuery(struct cmd_syndesc * as, void * rock)
 #ifdef AFS_DEMAND_ATTACH_FS
 	struct volume_hdr_LRU_stats * hdr_stats;
 #endif
-	struct DiskPartitionStats * vicep_stats;
+	struct DiskPartitionStats64 * vicep_stats;
     } sres;
 
     sres.ptr = res_buf;
@@ -1222,13 +1281,13 @@ print_vol_stats_general(VolPkgStats * stats)
 }
 
 static void
-print_vol_stats_viceP(struct DiskPartitionStats * stats)
+print_vol_stats_viceP(struct DiskPartitionStats64 * stats)
 {
-    printf("DiskPartitionStats = {\n");
-    printf("\tfree = %d\n", stats->free);
-    printf("\tminFree = %d\n", stats->minFree);
-    printf("\ttotalUsable = %d\n", stats->totalUsable);
-    printf("\tf_files = %d\n", stats->f_files);
+    printf("DiskPartitionStats64 = {\n");
+    printf("\tfree = %lld\n", stats->free);
+    printf("\tminFree = %lld\n", stats->minFree);
+    printf("\ttotalUsable = %lld\n", stats->totalUsable);
+    printf("\tf_files = %lld\n", stats->f_files);
 #ifdef AFS_DEMAND_ATTACH_FS
     printf("\tvol_list_len = %d\n", stats->vol_list_len);
 #endif
